"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const chalk_1 = __importDefault(require("chalk"));
const cross_spawn_1 = __importDefault(require("cross-spawn"));
const open_1 = __importDefault(require("open"));
class OpenBrowser {
    constructor(logger) {
        this.logger = logger;
        this.OSX_CHROME = 'google chrome';
        this.Actions = Object.freeze({
            NONE: 0,
            BROWSER: 1,
            SCRIPT: 2,
        });
    }
    open(url) {
        const { action, value, args } = this.getBrowserEnv();
        switch (action) {
            case this.Actions.NONE:
                return false;
            case this.Actions.SCRIPT:
                return this.executeNodeScript(value, url);
            case this.Actions.BROWSER:
                return this.startBrowserProcess(value, url, args);
            default:
                throw new Error('Not implemented.');
        }
    }
    getBrowserEnv() {
        const value = process.env.BROWSER;
        const args = process.env.BROWSER_ARGS ? process.env.BROWSER_ARGS.split(' ') : [];
        let action;
        if (!value) {
            action = this.Actions.BROWSER;
        }
        else if (value.toLowerCase().endsWith('.js')) {
            action = this.Actions.SCRIPT;
        }
        else if (value.toLowerCase() === 'none') {
            action = this.Actions.NONE;
        }
        else {
            action = this.Actions.BROWSER;
        }
        return { action, value, args };
    }
    executeNodeScript(scriptPath, url) {
        const extraArgs = process.argv.slice(2);
        const child = cross_spawn_1.default(process.execPath, [scriptPath, ...extraArgs, url], {
            stdio: 'inherit',
        });
        child.on('close', (code) => {
            if (code !== 0) {
                this.logger.info(chalk_1.default.red('The script specified as BROWSER environment variable failed.'));
                this.logger.info(`${chalk_1.default.cyan(scriptPath)} exited with code ${code}.`);
            }
        });
        return true;
    }
    startBrowserProcess(browser, url, args) {
        const shouldTryOpenChromiumWithAppleScript = process.platform === 'darwin' && (typeof browser !== 'string' || browser === this.OSX_CHROME);
        if (shouldTryOpenChromiumWithAppleScript) {
            const supportedChromiumBrowsers = [
                'Google Chrome Canary',
                'Google Chrome',
                'Microsoft Edge',
                'Brave Browser',
                'Vivaldi',
                'Chromium',
            ];
            for (const chromiumBrowser of supportedChromiumBrowsers) {
                try {
                    child_process_1.execSync(`ps cax | grep "${chromiumBrowser}"`);
                    child_process_1.execSync(`osascript openChrome.applescript "${encodeURI(url)}" "${chromiumBrowser}"`, {
                        cwd: __dirname,
                        stdio: 'ignore',
                    });
                    return true;
                }
                catch (err) {
                }
            }
        }
        if (process.platform === 'darwin' && browser === 'open') {
            browser = undefined;
        }
        if (typeof browser === 'string' && args.length > 0) {
            browser = [browser].concat(args);
        }
        try {
            const options = { app: browser, wait: false, url: true };
            open_1.default(url, options).catch(() => { });
            return true;
        }
        catch (err) {
            return false;
        }
    }
}
exports.OpenBrowser = OpenBrowser;
//# sourceMappingURL=open-browser.js.map